/*
    A Cycle Timer for StrongARM systems with 21285 bridges at 0xe10000,
      that would be NetWinders.
    Copyright (C) 1998   Jim Studt <jim@federated.com>

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

/*
    This is just a little test program for the nwtest.o file.
    You can run this, it won't terminate but will print a line for
    any suspicious output from nwtime's functions.  See below for an
    explanation of the output.
*/

#include <unistd.h>
#include <stdio.h>
#include <strings.h>
#include <stdlib.h>

#include "nwtimer.h"

int main( int argc, char **argv)
{
    struct nwtimer s,e;
    long long d;
    int i;

    for(i = 0;;i++) {
        /*
	** capature start time for our interval.  This is a very fast 
	** inline function (except the first time its called, but the
	** extra time is before the capture)
	*/
	record_time(&s);
	/*
	** Do all the work to be timed, in this example program we do 
	** nothing
	*/

	/*
	** capture the end time for our interval.  Very fast.
	*/
	record_time(&e);

	/*
	** extract a long long telling us how many cycles were in the 
	** interval.
	*/
	d = cycles_between(&s,&e);

	/*
	** if our interval was anomalous (remember we did nothing, it
	** should be pretty fast) then print it so we can see if the
	** algorithm is screwing up.  
	**
	** in practice, we should see abunch of these.  Any time an
	** interrupt happens to fire in our empty interval we will get
	** an interval large enough to trip this.  Doesn't mean the 
	** interrupt lasted this long, we could have been rescheduled.
	*/
	if ( d < 0 || d > 110) {
	    printf("%6d: %10d-%10d, %10d-%10d -> %qd\n", 
		   i, s.t1, e.t1, s.t2, e.t2, d);
	}
    }
    return 0;
}






