#ifndef _NWTIMER_H
#define _NWTIMER_H 1

/*
    A Cycle Timer for StrongARM systems with 21285 bridges at 0xe10000,
      that would be NetWinders.
    Copyright (C) 1998   Jim Studt <jim@federated.com>

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include <asm/hardware.h>

struct nwtimer {
    unsigned long t1;
    unsigned long t2;
};

/*
** compute the number of clock cycles between two struct nwtimer events
** capture the nwtimer events with record_time()
*/
extern unsigned long long cycles_between( const struct nwtimer *start,
					  const struct nwtimer *end);

/*
** compute the number of clock cycles from the last timer rollover
** to the time in t.
*/
extern unsigned long long cycles_to( const struct nwtimer *t);

/*
** internal function.  Do not use.  Just here for the following inline
** to use.
*/ 
extern volatile int *record_time_internal(void);

/*
** record a time marker.  very fast.  use this before and after your
** code, pass the two nwtimer structs into clock_cycles and see how
** many clock cycles it took
*/
extern inline void record_time( struct nwtimer *s)
{
    static volatile int *km = 0;

    if ( !km ) km = record_time_internal();
    {
	register volatile int *p = km + TIMER_2_VALUE/4;
	register unsigned long a,b;

	/*
	** We use the LoaD Multiple instruction with great cause here.
	** Were we to use two independent loads their could be an 
	** interrupt between them and that blows the calculations apart
	** in the later computations.  We REALLY need to have these two
	** registers read < 128 cycles apart.
	**
	** Note: ues I know I could just tell gcc we are using r0-r8,
	**       but then it pukes generating code for some functions.
	**       Our register use impacts the functiosn where we get inlined.
	**       This way we don't hurt so bad.  Just took two registers.
	*/
	asm ( "stmfd SP!,{r0-r8};"   /* save registers */
	      "ldmia %2,{r0-r8};"
	      "mov r9,r0;"
	      "mov r10,r8;"
	      "ldmfd SP!,{r0-r8};"    /* restore registers */
	      "mov %0,r9;"
	      "mov %1,r10"
	      : "=r" (a), "=r" (b) : "r" (p) : "r9","r10" );
	s->t1 = a;
	s->t2 = b;
    }
}




#endif
